# Clay Ford (jcf2d)
# StatLab@UVa
# Linear Mixed-Effect Modeling with R
# Spring 2018

####################
# YOUR TURN SOLUTION
####################

# The following data come from Ch 5 of Linear Mixed Models by West et al.

# An experiment that examined nucleotide activation in 5 different rats. Each
# rat was subjected to two different treatments (basal vs carbachol) and then
# nucleotide activation was measured in 3 different brain regions. Therefore
# each rat has 6 different measures.

URL2 <- "http://people.virginia.edu/~jcf2d/workshops/LMEinR/ratbrain.csv"
ratbrain <- read.csv(URL2)
head(ratbrain, n = 12)

# There is variation within rats, and variation between rats.

# line graphs for each rat by region within treatment
ggplot(ratbrain, aes(x = region, y = activate, group = animal)) +
  geom_point() +
  geom_line() +
  facet_wrap(~ treatment)

# interaction plot
interaction.plot(x.factor = ratbrain$region,
                 trace.factor = ratbrain$treatment, 
                 response = ratbrain$activate)

# interaction plot with ggplot
ggplot(ratbrain, aes(x = region, y = activate, color = treatment, group = treatment)) +
  stat_summary(fun.y = mean, geom = "line")

# Model activate as a function of treatment and region.
# Do we need an interaction?
# Do we need random effects for treatment? for region?
# Recall: the ||-syntax works only for numeric predictors; don't try it for this data.

rb0 <- lmer(activate ~ region + treatment + (1 | animal), data = ratbrain)
summary(rb0, corr = F)
plot(allEffects(rb0))

rb1 <- lmer(activate ~ region * treatment + (1 | animal), data = ratbrain)
summary(rb1, corr = F)
plot(allEffects(rb1))

rb2 <- lmer(activate ~ region * treatment + (treatment | animal), data = ratbrain)
summary(rb2, corr = F)
plot(allEffects(rb2))

rb3 <- lmer(activate ~ region * treatment + (region + treatment | animal), data = ratbrain)
summary(rb3, corr = F)
plot(allEffects(rb3))

anova(rb0, rb1, rb2, rb3, refit = F)

# rb2 appears to be the best of the candidate models

# Diagnostics
# residual vs. fitted values
plot(rb2)

# residuals by subjects (boxplots)
plot(rb2, animal ~ resid(.))
# bit of an outlier on animal 1 (R111397)

# check normality of residuals
qqnorm(resid(rb2))

# check constant variance of random effects
plot(ranef(rb2))

# check normality of random effects
lattice::qqmath(ranef(rb2))
# Not bad considering we only have 5 animals

# Catepillar plot
lattice::dotplot(ranef(rb2))

# visual check of model fit
plot(rb2, activate ~ fitted(.) | animal, abline = c(0,1))


# bootstrap model predictions with percentile CIs
nd <- expand.grid(treatment = levels(ratbrain$treatment),
                  region = levels(ratbrain$region))
b.out <- bootMer(x = rb2, FUN = function(x)predict(x, newdata = nd, 
                                                   re.form=NA),
                 .progress = "txt", nsim = 1000)
colnames(b.out$t) <- apply(nd, 1, paste, collapse = " ")
apply(b.out$t,2,quantile, probs = c(0.025, 0.975))

# Effect plot with bootstrap CIs
# transpose CI using t()
boot.cis <- t(apply(b.out$t,2,quantile, probs = c(0.025, 0.975)))
# place percentile CIs in data frame with newdata and predictions
effPred <- cbind(activate = predict(rb2, newdata = nd, re.form = NA),
                 nd,
                 lower = boot.cis[,1],
                 upper = boot.cis[,2])
# create effect plot in ggplot2
ggplot(effPred, aes(x = region, y = activate, group = 1)) +
  geom_point() + 
  geom_line() +
  geom_errorbar(aes(ymin = lower, ymax = upper), width = 0.1) +
  facet_wrap(~treatment) +
  ggtitle("region*treatment effect plot")

# Not much different from the effects package default plot with Normal SEs
plot(allEffects(rb2))
